!**************************************************************************************************
! Subroutine/Function/Program | SplitBufr V.1.1
! Author                      | F. Guillaume (CNRM/GMAP/OBS)
! Creation Date               | 25/04/2016
!--------------------------------------------------------------------------------------------------
! Description : explode metaBUFR file into single ones, eventually 1 per subset.  
!--------------------------------------------------------------------------------------------------
! Global Variables : 
!--------------------------------------------------------------------------------------------------
! History :
!           25/04/2016 : first release
!           22/06/2016 : allow to specify particular sequence of BUFR and/or subsets to split
!**************************************************************************************************

PROGRAM SplitBufr

use            :: parkind1,     only : jpim, jpib, jprb
use            :: bufr_module
use            :: Metabufr_module

implicit none

 character(len=128)              :: InputFile, OutputFile, chaine
 character(len=128),dimension(8) :: Args
 character(len=1)                :: Option
 integer(kind=jpim)              :: iret, NbSubset, NbFic, NbArgs
 integer(kind=jpim)              :: length,i,pos
 integer(kind=jpim)              :: FirstFile, LastFile, FirstSubset, LastSubset

! initialization
call bufr_alloc()

iret         = 0
nbsubset     = 1
FirstFile    = 0
LastFile     = 0
FirstSubset  = 0
LastSubset   = 0
Args(:)      = ''
Option       = ''
InputFile    = ''
OutputFile   = ''

NbArgs = command_argument_count()
if (NbArgs > size(Args)) call erreur("***** ERROR : too many arguments")
do i=1, NbArgs
  call get_command_argument(i,Args(i),length,iret)
enddo

i = 1
do while (i <= NbArgs)
  select case (trim(Args(i)))
  case ('-f') ! numeros des fichiers a traiter
    chaine = Args(i+1)
    pos = index(chaine,'-')
    if (pos > 0) then
      read(chaine(1:pos-1),*,iostat=iret) FirstFile
      if (iret /= 0) call erreur("***** ERROR : bad values in -f option")
      read(chaine(pos+1:len(trim(chaine))),*,iostat=iret) LastFile
      if (iret /= 0) call erreur("***** ERROR : bad values in -f option")
      if (FirstFile > LastFile .or. FirstFile <= 0) call erreur("***** ERROR : bad values in -f option")
    else
      read(chaine,*,iostat=iret) FirstFile
      if (iret /= 0 .or. FirstFile <= 0) call erreur("ERROR : bad values in -f option")
      LastFile = FirstFile
    endif
    i = i+2
  case ('-s') ! numeros des subsets à traiter
    chaine = Args(i+1)
    pos = index(chaine,'-')
    if (pos > 0) then
      read(chaine(1:pos-1),*,iostat=iret) FirstSubset
      if (iret /= 0) call erreur("***** ERROR : bad values in -s option")
      read(chaine(pos+1:len(trim(chaine))),*,iostat=iret) LastSubset
      if (iret /= 0) call erreur("***** ERROR : bad values in -s option")
      if (FirstSubset > LastSubset .or. FirstSubset <= 0) call erreur("***** ERROR : bad values in -s option")
    else
      read(chaine,*,iostat=iret) FirstSubset
      if (iret /= 0 .or. FirstSubset < 0) call erreur("***** ERROR : bad values in -s option")
      LastSubset = FirstSubset
    endif
    i = i+2
    Option = 's'
  case ('-i')
    InputFile = Args(i+1)
    i = i+2
  case ('-o')
    OutputFile = Args(i+1)
    i = i+2
  case ('-c')
    Option = 'c'
    i = i+1
  case default
    call erreur("***** ERROR : bad arguments")
  end select
enddo

select case (Option)
case ('s')
  if (len(trim(OutputFile)) > 0) then
    call SplitSubsets(InputFile,OutputFile,FirstFile,LastFile,FirstSubset,LastSubset)
  else
    call erreur("***** ERROR : OutputFile not specified")
  endif
case ('c')
  call CountBufr(InputFile,NbFic,NbSubset)
  if (NbFic > 999999) then
    write(*,'("WARNING : too many single bufr files contained in input file (>999999).")')
  endif
  if ((NbSubset/NbFic) > 9999) then
    write(*,'("WARNING : too many subsets per file (>9999).")')
  endif
  write(*,'(">> ",I6.6," files are contained in ",A,".")') NbFic,trim(adjustl(InputFile))
  write(*,'(">> ",I10.10," subsets found.")') NbSubset
case default
  if (len(trim(OutputFile)) > 0) then
    call SplitFile(InputFile,OutputFile,FirstFile,LastFile)
  else
    call erreur("***** ERROR : OutputFile not specified")
  endif
end select

END PROGRAM SplitBufr
